@@ -5,10 +5,10 @@ use warnings;
 use GraphQL::Schema;
 use GraphQL::Debug qw(_debug);
 use JSON::Validator::OpenAPI;
+use OpenAPI::Client;
 
-our $VERSION = "0.01";
+our $VERSION = "0.02";
 use constant DEBUG => $ENV{GRAPHQL_DEBUG};
-my $validator = JSON::Validator::OpenAPI->new; # singleton
 
 my %TYPEMAP = (
   string => 'String',
@@ -48,11 +48,11 @@ sub field_resolver {
   return $property // die "OpenAPI.resolver could not resolve '$field_name'\n"
     if ref $root_value eq 'HASH' or !$root_value->can($field_name);
   return $root_value->$field_name($args, $context, $info)
-    if !UNIVERSAL::isa($root_value, 'DBIx::Class::Core');
-  # dbic search
-  my $rs = $root_value->$field_name;
-  $rs = [ $rs->all ] if $info->{return_type}->isa('GraphQL::Type::List');
-  return $rs;
+    if !UNIVERSAL::isa($root_value, 'OpenAPI::Client');
+  # call OAC method
+  my $got = $root_value->$field_name($args);
+  DEBUG and _debug('OpenAPI.resolver(got)', $got->res->json);
+  $got->res->json;
 }
 
 sub _trim_name {
@@ -305,11 +305,12 @@ sub _kind2name2endpoint {
 }
 
 sub to_graphql {
-  my ($class, $spec) = @_;
-  my $dbic_schema_cb = undef;
-  my $openapi_schema = $validator->schema($spec)->schema;
+  my ($class, $spec, $app) = @_;
+  my %appargs = (app => $app) if $app;
+  my $openapi_schema = JSON::Validator::OpenAPI->new(
+    %appargs
+  )->schema($spec)->schema;
   my $defs = $openapi_schema->get("/definitions");
-  my %root_value;
   my @ast;
   my (
     %name2type,
@@ -339,15 +340,6 @@ sub to_graphql {
     fields => {
       map {
         my $name = $_;
-        $root_value{$name} = sub {
-          my ($args, $context, $info) = @_;
-          DEBUG and _debug('OpenAPI.root_value', );
-          [
-            $dbic_schema_cb->()->resultset($name)->search(
-              +{ map { ("me.$_" => $args->{$_}) } keys %$args },
-            )
-          ];
-        };
         (
           $name => $kind2name2endpoint->{query}{$name},
         )
@@ -360,14 +352,6 @@ sub to_graphql {
     fields => {
       map {
         my $name = $_;
-        $root_value{$name} = sub {
-          my ($args, $context, $info) = @_;
-          [
-            $dbic_schema_cb->()->resultset($name)->search(
-              +{ map { ("me.$_" => $args->{$_}) } keys %$args },
-            )
-          ];
-        };
         (
           $name => $kind2name2endpoint->{mutation}{$name},
         )
@@ -376,7 +360,7 @@ sub to_graphql {
   };
   +{
     schema => GraphQL::Schema->from_ast(\@ast),
-    root_value => \%root_value,
+    root_value => OpenAPI::Client->new($spec, %appargs),
     resolver => \&field_resolver,
   };
 }
@@ -411,125 +395,31 @@ GraphQL::Plugin::Convert::OpenAPI - convert OpenAPI schema to GraphQL schema
 =head1 DESCRIPTION
 
 This module implements the L<GraphQL::Plugin::Convert> API to convert
-a L<JSON::Validator::OpenAPI> to L<GraphQL::Schema> etc.
+a L<JSON::Validator::OpenAPI> specification to L<GraphQL::Schema> etc.
 
-=head2 Example
+It uses, from the given API spec:
 
-Consider this minimal data model:
+=over
 
-  blog:
-    id # primary key
-    articles # has_many
-    title # non null
-    language # nullable
-  article:
-    id # primary key
-    blog # foreign key to Blog
-    title # non null
-    content # nullable
+=item * the given "definitions" as output types
 
-=head2 Generated Output Types
+=item * the given "definitions" as input types when required for an
+input parameter
 
-These L<GraphQL::Type::Object> types will be generated:
+=item * the given operations as fields of either C<Query> if a C<GET>,
+or C<Mutation> otherwise
 
-  type Blog {
-    id: Int!
-    articles: [Article]
-    title: String!
-    language: String
-  }
+=back
 
-  type Article {
-    id: Int!
-    blog: Blog
-    title: String!
-    content: String
-  }
-
-  type Query {
-    blog(id: [Int!]!): [Blog]
-    article(id: [Int!]!): [Blog]
-  }
-
-Note that while the queries take a list, the return order is
-undefined. This also applies to the mutations. If this matters, request
-the primary key fields and use those to sort.
-
-=head2 Generated Input Types
-
-Different input types are needed for each of CRUD (Create, Read, Update,
-Delete).
-
-The create one needs to have non-null fields be non-null, for idiomatic
-GraphQL-level error-catching. The read one needs all fields nullable,
-since this will be how searches are implemented, allowing fields to be
-left un-searched-for. Both need to omit primary key fields. The read
-one also needs to omit foreign key fields, since the idiomatic GraphQL
-way for this is to request the other object, with this as a field on it,
-then request any required fields of this.
-
-Meanwhile, the update and delete ones need to include the primary key
-fields, to indicate what to mutate, and also all non-primary key fields
-as nullable, which for update will mean leaving them unchanged, and for
-delete is to be ignored.
-
-Therefore, for the above, these input types (and an updated Query,
-and Mutation) are created:
-
-  input BlogCreateInput {
-    title: String!
-    language: String
-  }
-
-  input BlogSearchInput {
-    title: String
-    language: String
-  }
-
-  input BlogMutateInput {
-    id: Int!
-    title: String
-    language: String
-  }
-
-  input ArticleCreateInput {
-    blog_id: Int!
-    title: String!
-    content: String
-  }
-
-  input ArticleSearchInput {
-    title: String
-    content: String
-  }
-
-  input ArticleMutateInput {
-    id: Int!
-    title: String!
-    language: String
-  }
-
-  type Mutation {
-    createBlog(input: [BlogCreateInput!]!): [Blog]
-    createArticle(input: [ArticleCreateInput!]!): [Article]
-    deleteBlog(input: [BlogMutateInput!]!): [Boolean]
-    deleteArticle(input: [ArticleMutateInput!]!): [Boolean]
-    updateBlog(input: [BlogMutateInput!]!): [Blog]
-    updateArticle(input: [ArticleMutateInput!]!): [Article]
-  }
-
-  extends type Query {
-    searchBlog(input: BlogSearchInput!): [Blog]
-    searchArticle(input: ArticleSearchInput!): [Article]
-  }
+The queries will be run against the spec's server.  If the spec starts
+with a C</>, and a L<Mojolicious> app is supplied (see below), that
+server will instead be the given app.
 
 =head1 ARGUMENTS
 
-To the C<to_graphql> method: a code-ref returning a L<DBIx::Class::Schema>
-object. This is so it can be called during the conversion process,
-but also during execution of a long-running process to e.g. execute
-database queries, when the database handle passed to this method as a
-simple value might have expired.
+To the C<to_graphql> method: a URL to a specification, or a filename
+containing a JSON specification, of an OpenAPI v2. Optionally, a
+L<Mojolicious> app can be given as the second argument.
 
 =head1 PACKAGE FUNCTIONS
 
